*this do-file runs a demonstration to be used in the presentation for the Stata users'
*group meeting, 17th-18th May 2005

*Date: 140405

***********************

/*
The contents of the talk will include:
1) a quick introduction to the dataset which will be part of the Ashram dataset
2)Finding, counting and removing duplicated data and other multiple entries;
3)Recap of uses of the merge command
4)Generating a moving target variable

*/

clear
set mem 10m
cd "H:\statausers_may05\public"
pause on
set more on



**********
*1) introduction to the dataset; the data have been doctored!
use 041217_demographic.dta, clear
desc
pause
br
use 041217_medicalhx.dta, clear
desc
pause
br
pause
use 041217_demographic.dta, clear


**********
*2)Finding, counting and removing duplicated data and other multiple entries
pause

*We can use the "duplicates" commands to identify and look at duplicates
duplicates report
pause
duplicates browse
duplicates tag, gen(duplicates)
tab duplicates
pause
drop duplicates
*These are very useful labour-saving commands! - but it may also be useful to be able to identify
*duplicates by hand
pause

*By hand:
*First we need to work out which item(s) in the data may have been duplicated - the most likely is 
*id number
pause
egen n=count(id), by(id)
tab n
pause
br if n>1
pause
*we need to delete the identical entries
pause
br if n==3
pause
*we can use the duplicates command to do this
duplicates drop
pause
*now we have to establish which of the two entries with n=2 is the correct one
pause
br if n==2
pause
*we have decided that we need to keep the second of these two entries; this means that we have
*to identify which one to keep
*we order the duplicated observations:
bysort n: gen order=_n if n==2
pause
br if n==2
pause
drop if n==2&order==1
*now checking that there are no more duplicated id numbers:
egen n2=count(id), by(id)
tab n2
pause
*there are no more duplicated id numbers
*the same procedure can be repeated for other fields which should be unique to each subject
drop n n2 order
pause
sort id
save 041219_demographic2.dta, replace

****************

****************
*3)Recap of merging data and uses of the merge command

*The merge command allows us to join datasets together by some common variable

use 041219_demographic2.dta, clear
merge id using 041217_medicalhx.dta

*A Stata system variable, _merge, is generated here. This is an extremely useful variable as it
*tells you which dataset each of your observations belongs to. It can be used exactly like any
*other Stata variable to list or browse parts of the datasets according to where the observations
*are from.
pause

tab _merge
pause
br if _merge==1
br if _merge==2
br if _merge==3
pause

/* This command puts the datasets together by id. Observations not occuring in both datasets have
missing values for the variables in the datasets to which they do not belong. In this way each 
individual has a complete demographic and medical record where both sets of data are non-missing.

If we just want to superimpose the medical data on the demographic data we do not specify a variable
upon which to merge:

use 041219_demographic2.dta, clear
merge using 041217_medicalhx.dta

However in most cases we wish to put the data together according to some identifier as we have shown.

*/
pause


*We may be surprised that some people are lacking demographic data and wish to identify which ones
*they are:
l id if _merge==2, noobs sep(0)


pause

*if we require a dataset containing only the people who have complete data:
drop if _merge<3


*******

*We can multiply merge datasets as well.
*This is an easy and quick way of putting lots of datasets together and finding out which 
*observations are common to which datasets.


pause

*We have several pages of medical questionnaire data moved across from an Access database. These
*need to be merged into one large dataset.

*The data look like this:
use "050308_p1.dta", clear
pause
br
use "050308_p2.dta", clear
br

*To multiply merge the data:
pause
use "050308_p1.dta", clear
merge id using 050308_p2 050308_p3 050308_p4 050308_p5 050308_p6 050308_p7
pause
summ _merge*
tab _merge
pause
tab _merge1
pause
tab _merge2
pause
tab _merge3
pause
tab _merge4
pause
tab _merge5
pause
tab _merge6
pause
*the _merge variable is the same as above
*the other merge variables represent the merge contribution to the final dataset at each stage of the
*merge; for example _merge1 represents the contribution of datasets 1 and 2 to the final dataset; 
*0 denotes that the observation is not in both datsets while 1 denotes an observation in both
*datasets. Note that the total equals the total number of observations in the final dataset.

pause

**************

**************
*4) the generation of a moving target variable - "changeyear"

/*
Here we need to identify the year, counting backwards from 2002, in which the recorded concentration
of chemical in each town changed.

*/
use archivedmeasures, clear

br
*definition of changeyear variable; first of all we need to identify those towns for which 
*the change occurred between 2001 and 2002. We can't do this in a "for" loop because the line
*contains the "gen" command

gen changeyear=2002 if y2002~=y2001
pause
/* Now we need to replace the changeyear variable with a year for which the concentration
of that year is not equal to the concentration of the previous year. We can do this using
the "forval" command and working backwards
*/
pause


forval num=2001(-1)1983 {
	local i=`num'-1
	qui replace changeyear=`num' if y`num'~=y`i'&changeyear==.
	}


*finally we move the change point to midway between the years so that it is clear when the change
*occurred


qui replace changeyear=changeyear-.5
pause
br if changeyear<.&y2002>.5





